package sample.nxnj;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.ModelAndView;

import devpia.dextuploadnj.Environment;
import devpia.dextuploadnj.FileItem;
import devpia.dextuploadnj.support.spring.DEXTUploadNJFileDownloadView;
import devpia.dextuploadnj.support.spring.DEXTUploadNJStreamDownloadView;

@Controller
public class FileServiceController {
	
	/**
	 * dextnj-servlet.xml에 선언된 환경 설정 객체입니다.
	 * Spring 프레임워크에 의해서 주입되었습니다.
	 */
	@Autowired
	private Environment dextuploadnjEnvironment;
	
	/**
	 * 단순 파일 업로드를 처리합니다.
	 * DEXTUploadNXRequest 클래스는 제품 라이브러리에 포함된 클래스가 아닙니다.
	 * Spring은 모델 바인딩 기능을 가지고 있으므로 DEXTUploadNX로부터 전달되는 폼 정보를 속성으로 갖는 예시 클래스입니다.
	 * @param nx DEXTUploadNX 제품으로 업로드를 하는 경우 DEXTUploadNXRequest 객체로 업로드 정보를 받을 수 있습니다.
	 * @param response 클라이언트로 응답 데이터를 전달하기 위해 HttpServletResponse 객체가 필요합니다.
	 * @throws IOException
	 */
	@RequestMapping(value = "/service/simpleupload.do")
	public void simpleupload(DEXTUploadNXRequest nx, HttpServletResponse response) throws IOException {
		FileItem item = null;
		StringBuffer sb = new StringBuffer();
		
		for (MultipartFile file : nx.getFiles()) {
			item = (FileItem)file;
			if (item.isEligibleFile()) {
				item.save();
				
				sb.append(String.format("name=%1$s, size=%2$d, path=%3$s\n",
					item.getFilename(),
					item.getFileSize(),
					item.getLastSavedFilePath()));
			}
		}		
				
		response.setCharacterEncoding("UTF-8");
		response.getWriter().write(sb.toString());
	}
	
	/**
	 * 파일 업로드를 처리합니다.
	 * 가상 파일 및 삭제된 가상 파일 정보도 함께 처리합니다.
	 * @param nx DEXTUploadNX 제품으로 업로드를 하는 경우 DEXTUploadNXRequest 객체로 업로드 정보를 받을 수 있습니다.
	 * @param response 클라이언트로 응답 데이터를 전달하기 위해 HttpServletResponse 객체가 필요합니다.
	 * @throws IOException
	 */
	@RequestMapping(value = "/service/defaultupload.do")
	public void defaultupload(DEXTUploadNXRequest nx, HttpServletResponse response) throws IOException {		

		FileItem item = null;
		StringBuffer sb = new StringBuffer();
		
		for (String vid : nx.getVirtualFileIds()) {
			// 가상 파일의 인덱스 정보를 얻을 수 있습니다.
			sb.append(String.format("type=%1$s, virtualIndex=%2$s\n", DEXTUploadNXFileType.Virtual, vid));
		}	
		
		for (String vid : nx.getVirtualDeletedFileIds()) {
			// 삭제된 가상 파일의 인덱스 정보를 얻을 수 있습니다.
			// 삭제된 파일 정보를 가지고 실제 파일을 삭제하는 코드를 구현하면 됩니다.
			// 일반적으로 실제 파일 삭제는 가장 마지막에 수행하는 것이 좋습니다.
			sb.append(String.format("type=%1$s, virtualIndex=%2$s\n", DEXTUploadNXFileType.Deleted, vid));
		}
		
		for (MultipartFile file : nx.getFiles()) {
			item = (FileItem)file;
			if (item.isEligibleFile()) {
				item.save();
				
				sb.append(String.format("type=%1$s, name=%2$s, size=%3$d, path=%4$s\n",
					DEXTUploadNXFileType.New,
					item.getFilename(),
					item.getFileSize(),
					item.getLastSavedFilePath()));
			}
		}
				
		response.setCharacterEncoding("UTF-8");
		response.getWriter().write(sb.toString());
	}
	
	/**
	 * 파일 업로드를 처리합니다.
	 * DEXTUploadNX에서 전달된 폴더 정보를 기반으로 디렉터리를 생성합니다.
	 * @param nx DEXTUploadNX 제품으로 업로드를 하는 경우 DEXTUploadNXRequest 객체로 업로드 정보를 받을 수 있습니다.
	 * @param response 클라이언트로 응답 데이터를 전달하기 위해 HttpServletResponse 객체가 필요합니다.
	 * @throws IOException
	 */
	@RequestMapping(value = "/service/folderupload.do")
	public void folderupload(DEXTUploadNXRequest nx, HttpServletResponse response) throws IOException {		

		FileItem item = null;
		File dir = null;
		StringBuffer sb = new StringBuffer();
		
		// 파일을 저장하는 루트 디렉터리 경로를 가져옵니다.
		String root = dextuploadnjEnvironment.getDefaultRepository();
		
		for (int i = 0; i < nx.getFiles().size(); i++) {
			
			// 루트 경로부터 상대 경로를 생성합니다.
			// 클라이언트와 서버는 파일 시스템이 다를 수 있으므로
			// 경로를 서버 파일 시스템 맞도록 변경해야 합니다.
			dir = new File(root, nx.getFolderPathes().get(i).replace("\\", File.separator));
			if (!dir.exists()) {
				dir.mkdir();
			}
			
			item = (FileItem)nx.getFiles().get(i);
			
			if (item.isEligibleFile()) {
				// 파일이 저장되는 위치를 변경하여 저장합니다.
				item.save(dir.getAbsolutePath());
				
				sb.append(String.format("name=%1$s, size=%2$d, path=%3$s\n",
					item.getFilename(),
					item.getFileSize(),
					item.getLastSavedFilePath()));
			}
		}
		
		for (String path : nx.getEmptyFolderPathes()) {
			// 파일이 없는 폴더의 경로를 가지고 옵니다.
			// 루트 경로부터 상대 경로를 생성합니다.
			dir = new File(root, path.replace("\\", File.separator));
			
			if (!dir.exists()) {
				dir.mkdir();
			}
			sb.append(String.format("path=%1$s\n", dir.getAbsolutePath()));
		}
				
		response.setCharacterEncoding("UTF-8");
		response.getWriter().write(sb.toString());
	}
	
	/**
	 * 파일 업로드를 처리합니다.
	 * DEXTUploadNX에서 User Data를 얻습니다.
	 * @param nx DEXTUploadNX 제품으로 업로드를 하는 경우 DEXTUploadNXRequest 객체로 업로드 정보를 받을 수 있습니다.
	 * @param response 클라이언트로 응답 데이터를 전달하기 위해 HttpServletResponse 객체가 필요합니다.
	 * @throws IOException
	 */
	@RequestMapping(value = "/service/userdataupload.do")
	public void userdataupload(DEXTUploadNXRequest nx, HttpServletResponse response) throws IOException {		

		FileItem item = null;
		String userdata = null;
		StringBuffer sb = new StringBuffer();
		
		for (int i = 0; i < nx.getFiles().size(); i++) {
			item = (FileItem)nx.getFiles().get(i);
			if (item.isEligibleFile()) {
				item.save();
				
				// User Data는 다음과 같습니다. => a[SPLIT]123[SPLIT]b[SPLIT][EMPTY]c[SPLIT]QWE[SPLIT]
				// 실제 User Data를 사용하려면 [SPLIT] 문자열을 사용하여 분해해야 합니다.
				userdata = nx.getUserData().get(i);
				
				sb.append(String.format("name=%1$s, userdata=%2$s\n", item.getFilename(), userdata));
			}
		}
				
		response.setCharacterEncoding("UTF-8");
		response.getWriter().write(sb.toString());
	}
	
	/**
	 * 파일 업로드를 처리합니다.
	 * DEXTUploadNX에서 Form 요소들을 얻습니다.
	 * @param nx DEXTUploadNX 제품으로 업로드를 하는 경우 DEXTUploadNXRequest 객체로 업로드 정보를 받을 수 있습니다.
	 * @param request HttpServletRequest 객체
	 * @param response 클라이언트로 응답 데이터를 전달하기 위해 HttpServletResponse 객체가 필요합니다.
	 * @throws IOException
	 */
	@RequestMapping(value = "/service/formupload.do")
	public void formupload(DEXTUploadNXRequest nx, HttpServletRequest request, HttpServletResponse response) throws IOException {		

		FileItem item = null;
		String paramName = null;
		StringBuffer sb = new StringBuffer();
		StringBuffer values = null;
		
		@SuppressWarnings("unchecked")
		Enumeration<String> pNames = (Enumeration<String>)request.getParameterNames();
		while (pNames.hasMoreElements()) {
			paramName = pNames.nextElement();
			// DEXTUploadNX로 시작하는 폼 이름을 제외한 나머지
			if (!paramName.startsWith("DEXTUploadNX")) {
				values = new StringBuffer();
				for (String v : request.getParameterValues(paramName))
					values.append(v.concat(";"));
				sb.append(String.format("type=파라미터 name=%1$s value=%2$s\n", paramName, values.toString()));
			}
		}
		
		for (int i = 0; i < nx.getFiles().size(); i++) {
			item = (FileItem)nx.getFiles().get(i);
			if (item.isEligibleFile()) {
				item.save();
				
				sb.append(String.format("type=파일 name=%1$s\n", item.getFilename()));
			}
		}
				
		response.setCharacterEncoding("UTF-8");
		response.getWriter().write(sb.toString());
	}
	
	/**
	 * 파일 업로드를 처리합니다.
	 * DEXTUploadNX에서 User Data를 얻습니다.
	 * @param nx DEXTUploadNX 제품으로 업로드를 하는 경우 DEXTUploadNXRequest 객체로 업로드 정보를 받을 수 있습니다.
	 * @param response 클라이언트로 응답 데이터를 전달하기 위해 HttpServletResponse 객체가 필요합니다.
	 * @throws IOException
	 */
	@RequestMapping(value = "/service/exifupload.do")
	public void exifupload(DEXTUploadNXRequest nx, HttpServletResponse response) throws IOException {		

		FileItem item = null;
		String exif = null;
		StringBuffer sb = new StringBuffer();
		
		for (int i = 0; i < nx.getFiles().size(); i++) {
			item = (FileItem)nx.getFiles().get(i);
			if (item.isEligibleFile()) {
				item.save();
				
				// EXIF는 다음과 같습니다. => a[SPLIT]123[SPLIT]b[SPLIT][EMPTY]c[SPLIT]QWE[SPLIT]
				// 실제 EXIF를 사용하려면 [SPLIT] 문자열을 사용하여 분해해야 합니다.
				exif = nx.getExifData().get(i);
				
				sb.append(String.format("name=%1$s, exif=%2$s\n", item.getFilename(), exif));
			}
		}
				
		response.setCharacterEncoding("UTF-8");
		response.getWriter().write(sb.toString());
	}
	
	/**
	 * OROF 파일 업로드를 처리합니다.
	 * 이 메소드는 파일의 개수 + 1만큼 호출됩니다.
	 * @param nx DEXTUploadNX 제품으로 업로드를 하는 경우 DEXTUploadNXRequest 객체로 업로드 정보를 받을 수 있습니다.
	 * @param request HttpServletRequest 객체
	 * @param response 클라이언트로 응답 데이터를 전달하기 위해 HttpServletResponse 객체가 필요합니다.
	 * @throws IOException
	 */
	@SuppressWarnings("unused")
	@RequestMapping(value = "/service/orofupload.do")
	public void orofupload(DEXTUploadNXRequest nx, HttpServletRequest request, HttpServletResponse response) throws IOException {
		FileItem item = null;
		String paramName = null;
		StringBuffer sb = new StringBuffer();
		StringBuffer values = null;
		
		if (nx.hasFiles()) {
			// 파일이 존재하는 경우이며 파일의 개수만큼 코드를 수행할 것입니다.
			item = (FileItem)nx.getFiles().get(0);
			if (item.isEligibleFile()) {
				item.save();
				sb.append(String.format("type=파일 name=%1$s\n", item.getFilename()));
			}
		} else {
			// 파일이 없는 경우에는 DEXTUploadNX에서 마지막 요청을 한 것입니다.
			
			// HTML 폼 및 사용자 폼 데이터를 처리합니다.
			@SuppressWarnings("unchecked")
			Enumeration<String> pNames = (Enumeration<String>)request.getParameterNames();
			while (pNames.hasMoreElements()) {
				paramName = pNames.nextElement();
				// DEXTUploadNX로 시작하는 폼 이름을 제외한 나머지
				if (!paramName.startsWith("DEXTUploadNX")) {
					values = new StringBuffer();
					for (String v : request.getParameterValues(paramName))
						values.append(v.concat(";"));
					sb.append(String.format("type=파라미터 name=%1$s value=%2$s\n", paramName, values.toString()));
				}
			}
			
			for (String vid : nx.getVirtualFileIds()) {
				// 가상 파일 처리는 여기서 처리합니다.
			}	
			
			for (String vid : nx.getVirtualDeletedFileIds()) {
				// 삭제된 가상 파일은 여기서 처리합니다.
			}
			
			for (String path : nx.getEmptyFolderPathes()) {
				// 빈 폴더 처리를 수행합니다.
			}
		}	
				
		response.setCharacterEncoding("UTF-8");
		response.getWriter().write(sb.toString());
	}
	
	/**
	 * 파일 업로드를 처리합니다.
	 * DEXTUploadNX에서 별칭을 얻습니다.
	 * @param nx DEXTUploadNX 제품으로 업로드를 하는 경우 DEXTUploadNXRequest 객체로 업로드 정보를 받을 수 있습니다.
	 * @param response 클라이언트로 응답 데이터를 전달하기 위해 HttpServletResponse 객체가 필요합니다.
	 * @throws IOException
	 */
	@RequestMapping(value = "/service/aliasupload.do")
	public void aliasupload(DEXTUploadNXRequest nx, HttpServletResponse response) throws IOException {		

		FileItem item = null;
		String alias = null;
		StringBuffer sb = new StringBuffer();
		
		for (int i = 0; i < nx.getFiles().size(); i++) {
			item = (FileItem)nx.getFiles().get(i);
			if (item.isEligibleFile()) {
				item.save();
				
				// 별칭을 얻습니다.
				alias = nx.getAliases().get(i);
				
				sb.append(String.format("name=%1$s, alias=%2$s\n", item.getFilename(), alias));
			}
		}
				
		response.setCharacterEncoding("UTF-8");
		response.getWriter().write(sb.toString());
	}
	
	@RequestMapping(value = "/service/targetdownload.do")
	public ModelAndView targetdownload(@RequestParam(value = "key") String key, HttpServletRequest request, HttpServletResponse response) throws IOException {
		
		File target = null;
		String rootDirPath = request.getSession().getServletContext().getRealPath("files/attach");
		
		if (key.equals("VF1"))
			target = new File(rootDirPath, "첨부파일_이미지.gif");
		else if (key.equals("VF2"))
			target = new File(rootDirPath, "첨부파일_텍스트.txt");		
		else if (key.equals("VF3"))
			target = new File(rootDirPath, "DEXTENC_첨부파일_이미지.gif");		
		else if (key.equals("VF4"))
			target = new File(rootDirPath, "DEXTENC_첨부파일_텍스트.txt");	
		
		if (target != null && target.exists()) {
			DEXTUploadNJFileDownloadView dextnj = new DEXTUploadNJFileDownloadView(target);			
			return new ModelAndView(dextnj);
		} else {
			response.setCharacterEncoding("UTF-8");
			response.sendError(HttpServletResponse.SC_NOT_FOUND, "The target file does not found.");
			return null;
		}
	}
	
	@RequestMapping(value = "/service/streamdownload.do")
	public ModelAndView streamdownload(@RequestParam(value = "loop") int loop, HttpServletRequest request, HttpServletResponse response) throws Exception {
		
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < loop; i++) {
			sb.append("0123456789\r\n");
		}

		InputStream is = null;
		
		try {
			is = new ByteArrayInputStream(sb.toString().getBytes("UTF-8"));
			
			// 스트림 형식으로 다운로드 하기 위해서 DEXTUploadNJStreamDownloadView 객체를 생성합니다.
			DEXTUploadNJStreamDownloadView view = new DEXTUploadNJStreamDownloadView();
			
			// 입력 스트림을 설정합니다.
			view.setInputStream(is);
			view.setFilename("stream.txt");
			view.setMime("text/plain");
			view.setCharsetName("UTF-8");
			
			return new ModelAndView(view);
			
		} catch (Exception e) {
			throw e;
		} finally {
			if (is != null) is.close();
		}
	}
}
